from setuptools import setup

setup(
    name="elizabeth",
    version="0.0.1",
    author="Shuhei Kishi",
    author_email="marubontan@gmail.com",
    description=("Machine leaning pipeline"),
    license="MIT",
    url="https://github.com/marubontan/elizabeth",
    packages=['elizabeth']
)
